import {
	Game,
	TouchSprite,
	RandomInt,
	MyLabel,
	blinkItem,
	stopBlinkItem,
	asyncDelayTime,
	asyncTweenChange,
} from './Game';

import {
	Label,
	jelly,
	showPopParticle,
	tweenChange,
	rotateItem,
	scaleItem,
	delayCall,
	hideItem,
	showItem,
	moveItem,
	shake,
	ShapeRect,
} from './../Unit';

import TWEEN from '@tweenjs/tween.js';

import { defaultData } from './DefaultData';


export class MyGame extends Game {

	images = null;

	data = null;
	status = null;

	initData(data) {
		this.images = data.images;

		this.status = {};

		if (!data.data || Object.is(data.data, {}) || !data.data.tittle) {
			this.data = defaultData;
		} else {
			this.data = data.data;

			let imgUrlList = [];
			// this.data.cubes.forEach((cube) => {
			// 	imgUrlList.push(cube.image.url);
			// });

			let audioUrlList = [];
			// this.data.cubes.forEach((cube) => {
			// 	audioUrlList.push(...(cube.audios.flat(Infinity).filter(val => val !== '')));
			// });
			// audioUrlList.push(this.data.tittle.audio);

			this.preLoadData(imgUrlList, audioUrlList);
		}
	}

	preLoadData(imgUrlList, audioUrlList) {
		for (var i = 0; i < imgUrlList.length; ++i) {
			this._parent.addUrlToImages(imgUrlList[i]);
		}
		for (var i = 0; i < audioUrlList.length; ++i) {
			this._parent.addUrlToAudioObj(audioUrlList[i]);
		}
	}

	initView() {
		// 初始化背景
		this.initBg();

		// 初始化标题
		this.initTittle();

		// 初始化中间部分
		this.initMiddle();

		// 游戏开始
		this.gameStart();
	}

	bg = null;
	initBg() {
		this.removeChild(this.getChildByName('bg'));

		this.getFullScaleXY();

		let screenSize = this.getScreenSize();
		let defaultSize = this.getDefaultScreenSize();

		let bgSized = new TouchSprite();
		bgSized.init(this.images.get('Img_bg'));
		bgSized.anchorX = 0.5;
		bgSized.anchorY = 1;
		bgSized.setPosition(this._parent.canvasWidth / 2, this._parent.canvasHeight);
		bgSized.setScaleXY(this.getFullScaleXY());
		this.addChild(bgSized);

		// 背景
		let bg = new TouchSprite();
		bg.init(this.images.get('Img_bg'))
		bg.setPosition(this._parent.canvasWidth / 2, this._parent.canvasHeight / 2);
		bg.setScaleXY(this._parent.mapScale);
		bg.alpha = 0.5;
		bg.setName('bg');
		this.addChild(bg);

		this.bg = bg;

		const bgRect = new ShapeRect();
		bgRect.setSize(57, 65);
		bgRect.fillColor = '#f8c224';
		const sx = this._parent.canvasWidth / this._parent.canvasBaseW;
		bgRect.setScaleXY(sx);
		bgRect.x = 65 * sx;
		bgRect.alpha = 0;
		this._parent.renderArr.push(bgRect);
		this.bgRect = bgRect;
	}

	bgRect = null;
	getTittleBlockPositionX() {
		return this.bgRect.x + 28.5 * this.bgRect.scaleX;
	}

	initTittle() {
		// 标题字母背景
		let tittleWordBg = new TouchSprite();
		tittleWordBg.init(this.images.get('Img_tittleBg'));
		tittleWordBg.setScaleXY(this._parent.mapScale);
		tittleWordBg.setPosition(this.getTittleBlockPositionX(), 31 * this._parent.mapScale);
		this.addChild(tittleWordBg);

		// 标题字母
		let tittleWord = new MyLabel();
		tittleWord.fontSize = 48;
		tittleWord.fontName = 'BerlinSansFBDemi-Bold';
		tittleWord.fontColor = '#ab5b22';
		tittleWord.text = this.data.tittle.word;
		tittleWord.anchorX = 0.5;
		tittleWord.anchorY = 0.5;
		tittleWord.setPosition(0, 0);

		tittleWordBg.addChild(tittleWord);
		tittleWord.refreshSize();

		// 标题文字
		let questionLabel = new MyLabel();
		questionLabel.fontSize = 36;
		questionLabel.fontName = 'FuturaBT-Bold';
		questionLabel.fontColor = '#000000';
		questionLabel.text = this.data.tittle.text;
		questionLabel.anchorX = 0;
		questionLabel.anchorY = 0.5;
		questionLabel.setPosition(50, 0);

		questionLabel.addTouchBeganListener(() => {
			this.onClickTittle();
		});

		tittleWordBg.addChild(questionLabel);
		questionLabel.refreshSize();
	}

	initMiddle() {

		// 创建背景
		this.createMiddlebg();
		// 创建单词
		this.createWords();
		// 创建右下角的喇叭
		this.createSpeaker();

	}

	createMiddlebg() {
		// 背景
		const bookBg = new TouchSprite();
		bookBg.init(this.images.get(this.typeDiff('Img_whole_bg_type_A', 'Img_whole_bg_type_B')));
		this.bg.addChild(bookBg);
	}

	createWords() {
		const leftPosX = this.typeDiff(-500, -480);
		const rightPosX = this.typeDiff(100, 90);

		const topPosY = -170;
		const middlePosY = 0;
		const bottomPosY = this.typeDiff(165, 170);

		const positionList = [
			[leftPosX, topPosY],
			[leftPosX, middlePosY],
			[leftPosX, bottomPosY],
			[rightPosX, topPosY],
			[rightPosX, middlePosY],
			[rightPosX, bottomPosY],
		];
		this.data.words.forEach((word, idx) => {
			const questionLabel = new MyLabel();
			questionLabel.fontSize = this.typeDiff(48, 54);
			questionLabel.fontName = this.typeDiff('CenturyGothic', 'BerlinSansFBDemi-Bold');
			questionLabel.fontColor = '#000000';
			questionLabel.text = word;
			questionLabel.anchorX = 0;
			questionLabel.anchorY = 0.5;
			questionLabel.setPosition(...positionList[idx]);

			questionLabel.addTouchBeganListener(() => {
				this.onClickWords(idx);
			});

			this.bg.addChild(questionLabel);
			questionLabel.refreshSize();
		});
	}

	createSpeaker() {
		const speaker = new TouchSprite();
		speaker.init(this.images.get('Btn_play'));
		speaker.setPositionX((this.getScreenSize().width / 2 - 50));
		speaker.setPositionY((this.getScreenSize().height / 2 - 50));
		this.bg.addChild(speaker);
	}

	typeDiff(a, b) {
		if (this.data.showType === 'A') {
			return a;
		} else {
			return b;
		}
	}

	printCurrentStatus() {
		console.log(JSON.stringify(this.status));
	}

	onClickTittle() {
		this._parent.playAudio(this.data.tittle.audio);
	}

	onClickWords(idx) {
		// this._parent.playAudio(this.data.audios[idx]);
	}

	onClickWinWords(idx) {

	}

	gameStart() {
		this.playIncomeAudio();
	}

	playIncomeAudio() {
		this._parent.playAudio('audio_new_page');
	}
}
